<?php
/**
 * Election Results
 * ----------------
 *
 * Algorithm:
 *
 * While No winner
 *   Foreach vote:
 *     foreach(number of holders of this position)
 *       pref = 1
 *       while(vote[pref] is in redistribute list || in voted_for)
 *         pref++
 *       total[vote]++
 *       add vote[pref] to voted_for
 *   if(clear winner)
 *     set winner
 *   else
 *     exclude lowest
 */

$title = _("Election Results");
headers(array('title'=>$title,'menu'=>1,
	      'head_extras'=>'<script type="text/javascript" src="lib/sorttable.js"></script>',
	      )
	);

if(!$_GET['id']) {
  echo '<p class="error">'._('No Election ID specified.').'</p>';
  exit(0);
}

require_once('backend/election.inc');

$election = get_election($_GET['id']);

if($election['results']!='available'
   && !has_permission(get_member_id($_SESSION['user']),'election admin')) {
  echo '<p class="error">'._('Results are not available for this election.').'</p>';
  exit(1);
}

if($election['results']!='available'
   && has_permission(get_member_id($_SESSION['user']),'election admin')) {
  echo '<p class="error"><b>'._('These are not final results.').'</b> '
    ._('You are only seeing this page because you are an administrator.')
    .'</p>';
}


?>
<h2><?=_('Important Notice')?></h2>
  <p><?=_('These results are preliminary and have not been approved by the returning officer.')?></p>
  <h2><?=_('Election Results')?></h2>
  <p><?=_('These are results for ')?><?=htmlise($election['name'])?></p>
<?php
$result = $dbh->query("select count(distinct member_id) as count from election_vote where candidate_id in (select id from election_candidate where election_position_id in (select id from election_position where election_id=".$election['id']."))");
if(DB::isError($result)) {
  echo '<p class="error">'._('Error getting vote information.').'</p>';
  exit(1);
}
$row = $result->fetchrow(DB_FETCHMODE_OBJECT);
?>
<p><?php echo _('There were ').($row->count)._(' members who voted in this election.');?></p>

<?php


function get_result($position,$preference,$redistribute)
{
  global $dbh;
  $p = $position;

  $exclude='(';
  foreach($redistribute as $r)
    $exclude.=$r.',';
  $exclude = preg_replace('/,$/',')',$exclude);

  $sql = "SELECT count(member_id) as votes, candidate_id from election_vote";
  $sql.=" where preference=".$preference;
  $sql.=" and candidate_id in ";
  $sql.=" (select id from election_candidate";
  $sql.=" where election_position_id =".$dbh->quote($p).")";
  if($preference > 1)
  $sql.=" and member_id in (select member_id from election_vote where preference=".($preference-1)." and candidate_id in ".$exclude.")";
  $sql.=" group by candidate_id order by votes DESC";


  $result = $dbh->query($sql);

  if(DB::isError($result)) {
    echo '<p class="error">'._('Error getting vote counts.').'</p>';
    exit(1);
  }
  $votes =& $dbh->getAll($sql,array(),DB_FETCHMODE_ASSOC);

  return $votes;
}

function show_result($position,$votes,$totalvotes,$winner)
{
  global $dbh;

  $p = $position;
  echo '<table class="sortable" id="resultsfor'.htmlise($p).'">';
  echo '<thead>';
  echo '<tr>';
  echo '<th>Votes</th>';
  echo '<th>Candidate</th>';
  echo '</thead>';
  echo '<tbody>';

  foreach($votes as $member_id=>$numvotes) {
    $m = get_member($member_id);
    echo '<tr><td>'.htmlise($numvotes).'</td>';
    echo '<td>'.htmlise($m->first_name.' '.$m->last_name).' '
    .'(<a href="mailto:'.htmlise($m->email).'">'.htmlise($m->email)
    .'</a>)</td></tr>';
  }
  echo '</tbody>';
  echo '</table>';

  echo '<p>';
  printf(_("There were %u votes. A candidate needs %u votes to have a clear majority."),$totalvotes,(floor(($totalvotes/2)+1)));
  echo '</p>';

  if($winner==0)
    echo '<p>'._('There is no clear winner at the moment. We will redistribute preferences.').'</p>';
  else if($winner<0)
    echo '<p><b>'._('The number of candidates remaining equals the number of position holders. These candidates are the winners.').'</b></p>';
  else {
    echo '<p><b>'._('We have a winner by a clear majority: ');
    $m=get_member($winner);
    echo htmlise($m->first_name.' '.$m->last_name);
    echo '</b></p>';
  }
}

/*
 * While No winner
 *   Foreach vote:
 *     foreach(number of holders of this position)
 *       pref = 1
 *       while(vote[pref] is in redistribute list || in voted_for)
 *         pref++
 *       total[vote]++
 *       add vote[pref] to voted_for
 *   if(clear winner)
 *     set winner
 *   else
 *     exclude lowest
*/

$winners = array(); // array of member_id of position winners

foreach(array_keys($election['position']) as $p) {

  echo '<h3>'.htmlise($election['position'][$p]['name']).'</h3>';
  $result = $dbh->query("select count(id) as count from election_candidate where election_position_id=".$dbh->quote($p));
  $num4position = $result->fetchrow(DB_FETCHMODE_OBJECT);


  // reset redistribution
  $redist_human_read=array(); // cached member info
  $redistribute=array();      /* array of member_ids of candidates to redistribute votes for */
  unset($last_redistribute);


  if(count($winners)) {
    print "<p>"._('We are redistributing votes for the winners: ')."<ul>";
    foreach(array_keys($winners) as $r) {
      $m=get_member($r);
      print "<li>".htmlise($m->first_name.' '.$m->last_name)."</li>";
    }
    print "</ul></p>";

  }

  if(count($redistribute)) {
    print "<p>"._('We are redistributing votes for: ')."<ul>";
    foreach(array_keys($redistribute) as $r) {
      $m = get_member($r);
      print "<li>".htmlise($m->first_name.' '.$m->last_name)."</li>";
    }
    print "</ul></p>";
  }

  // move winners into redistribute
  foreach(array_keys($winners) as $w)
    $redistribute[$w] = true;
  
  
  do {
    if(isset($last_redistribute)) {
      $m=get_member($last_redistribute);
      show_result($p,$votes,$totalvotes,0);
      echo '<p>'._('We are redistributing prefrences for: ');
      echo htmlise($m->first_name.' '.$m->last_name);
      echo '</p>';
    }
    unset($last_redistribute);

    $totalvotes = 0;
    $votes = array();
    

    
    $numholders = $election['position'][$p]['num'];

    $sql = "select distinct member_id from election_vote where candidate_id in (select id from election_candidate where election_position_id=".$dbh->quote($p).")";
    $result=$dbh->query($sql);

    while($row = $result->fetchrow(DB_FETCHMODE_OBJECT)) { // foreach vote
      $voted_for = array();
      for($h=0;$h<$numholders;$h++) { // foreach (num holders of position)
	$pref = 0;
	do {
	  $pref++;
	  // get vote for this pref
	  $sql = "select election_vote.*, ";
	  $sql.=" election_candidate.member_id as candidate_member_id";
	  $sql.=" from election_vote,election_candidate";
	  $sql.=" where election_vote.candidate_id=election_candidate.id";
	  $sql.=" and candidate_id in ";
	  $sql.=" (select id from election_candidate where ";
	  $sql.=" election_position_id=".$dbh->quote($p).") and ";
	  $sql.=" election_vote.preference=".$pref." and ";
	  $sql.=" election_vote.member_id=".$dbh->quote($row->member_id);
	  $res = $dbh->query($sql);
	  $vote= $res->fetchrow(DB_FETCHMODE_OBJECT);
	} while($res->numRows() // don't repeat if we have no more prefs
		&& (isset($redistribute[$vote->candidate_member_id])
		    || isset($voted_for[$vote->candidate_member_id]))
		);
	if($res->numRows()!=1)
	  break;
	$votes[$vote->candidate_member_id]++;
	$totalvotes++;
	$voted_for[$vote->candidate_member_id]= true;
      }
    }

    arsort($votes);
    
    unset($winner);
    $lowest_votes = $totalvotes;
    $lowest_id = 0;
    foreach($votes as $m=>$n) {
      if($n<$lowest_votes) { // find lowest number of votes
	$lowest_votes=$n;
	$lowest_id=$m;
      }
      if($n>($totalvotes/2)) {// find winner
	$winner = $m;
	$winners[$m]=true;
      }
    }

    $redistribute[$lowest_id] = true;
       $last_redistribute = $lowest_id;

  }while(!isset($winner) && count(array_keys($votes))>$numholders);
  
  if(!isset($winner) && count(array_keys($votes))==$numholders)
    $winner=-1;

  show_result($p,$votes,$totalvotes,$winner);
}

?>

