<?php
$title = _("Vote in an Election");
headers(array('title'=>$title,'menu'=>1));

if(!$_GET['id']) {
  echo '<p class="error">'._('No Election ID specified.').'</p>';
  exit(0);
}

require_once('backend/election.inc');

$election = get_election($_GET['id']);

if($election['voting']!='open') {
  echo '<p class="error">'._('Voting is not open for this election.').'</p>';
  exit(1);
}

// Validate any vote
$votes = array();
$votedupe = false;
foreach($_POST as $p=>$v) {
    if(preg_match("/^p(\d+)m(\d+)$/",$p,$matches)) {
      if($votes[$matches[1]][$v]) {
	// already got a number
	$votedupe = true;
      } else if(is_int($v+0) && $v>0) {
	$votes[$matches[1]][$v] = $matches[2];
      }
    }
}

// check the numbering
$votenonseq = false;
foreach(array_keys($votes) as $p) {
  $num = 1;
  $k = array_keys($votes[$p]);
  sort($k);
  foreach($k as $n) {
    if($n!=$num) {
      $votenonseq = true;
    } else
      $num++;
  }
}

$voteform = array('id'=>array('type'=>'hidden',
			      'default'=>htmlise($_GET['id'])
			      ),
		  'dummy'=>array('type'=>'hidden',
				 'label'=>_("Numbers aren't sequential from 1"),
				 'valid'=>array(" ".var_export(!$votenonseq,true)
						=>_("Numbers aren't sequential"),
						),
				 ),
		  'dummy2'=>array('type'=>'hidden',
				  'label'=>_('Duplicate numbers in vote'),
				  'valid'=>array(
						 "  ".var_export(!$votedupe,true)
						 =>_('Duplicate numbers in vote'),
						 ),
				  ),
		  );

foreach($election['candidates'] as $c) {
  if($last_election_position_id != $c['election_position_id']) {
    $posset = array('type'=>'fieldset',
		    'legend'=>$election['position']
		    [$c['election_position_id']]
		    ['name']
		    .' ('
		    .$election['position'][$c['election_position_id']]['num']
		    .')',
		    'fields'=>array()
		    );
    $posfields = &$posset['fields'];
    $voteform['position'.$c['election_position_id'].'fieldset']
      = $posset;
  }
  $last_election_position_id=$c['election_position_id'];
  $m = get_member($c['member_id']);
  $posfields['p'.$c['election_position_id'].'m'.$c['id']]
    = array('type'=>'select',
	    'label'=>htmlise($m->first_name.' '.$m->last_name).' '
	        .'(<a href="mailto:'.htmlise($m->email).'">'.htmlise($m->email)
	    .'</a>)',
             'items'=> election_make_rank_list( $c['election_position_id'], $election['candidates'] ),
	    );
}
?>

<h2><?=_('Vote in an Election')?></h2>
  <p><?=_("Voting is preferential. Sequentially number (from 1) your preferences for each position. You don't have to express a preference for all candidates; if you leave a candidate blank they simply don't get any vote from you.")?></p>
  <p><?=_("If you have already voted, your existing votes will be deleted before these new ones are recorded.")?></p>
<?php
$errors = validate($voteform,$_POST);
print $errors;
if($errors || !isset($_POST['election_vote'])) {
  output_form($voteform,'election_vote','Vote','post',"&id=".urlencode($_GET['id']));
} else {
  // Process the form
  global $dbh;
  $result = $dbh->query("begin");
  if(DB::isError($result))
    echo '<p class="error">'._('Error recording vote.').'</p>';

  $result = $dbh->query("delete from election_vote where member_id=".$dbh->quote(get_member_id($_SESSION['user']))." and candidate_id in (select id from election_candidate where election_position_id in (select id from election_position where election_id=".$dbh->quote($_GET['id'])."))");
  if(DB::isError($result))
    echo '<p class="error">'._('Error recording vote.').'</p>';

  foreach(array_keys($votes) as $p) {
    foreach($votes[$p] as $pref=>$c) {
      sql_insert('election_vote',
		 array(
		       'member_id'=>get_member_id($_SESSION['user']),
		       'candidate_id'=>$c,
		       'preference'=>$pref)
		 );
    }
  }
  $result = $dbh->query("commit");
  if (DB::isError($result)) {
    echo '<p class="error">'._('An error occured while entering your votes.')
      .'</p>';
  } else {
    echo '<p>'._('Your vote has been recorded.').'</p>';

    $messagevars= array('MEMBER_EMAIL'=>$_SESSION['user'],
			'ORG_NAME'=>$org->name,
			'ORG_URL'=>$org->website,
			'MEMBERSHIP_ADMIN'=>$config['membership-admin'],
			'ELECTION'=>$election['name'],
			);

    site_message('election-vote', $messagevars);
  }
}
?>
