<?php
/**
 * Member List
 *
 * Displays a (server side) sortable table of current_memberships.
 *
 * AUDIT: 20041124(stewart) - htmlise() all database output
 *
 * @package memberlist
 * @author Stewart Smith
*/
$title = _("Membership List");
headers(array('title'=>$title,'menu'=>0));

if(!has_permission(get_member_id($_SESSION['user']),
		   $permissions_pages['memberlist']['activity'])) {
  exit("No permission");
}
$order_by = array('id' => 'id,first_name,middle_name,last_name',
		  'date_entered' => 'date_entered,first_name,middle_name,last_name',
		  'name' => 'first_name,middle_name,last_name',
		  'name_l' => 'last_name,middle_name,first_name',
		  'dob' => 'dob',
		  'sex' => 'sex',
		  'address' => 'country,state,suburb,address1,address2',
		  'home_phone' => 'phone_home',
		  'mobile_phone' => 'phone_mobile',
		  'email' => 'email'
);
$sql = "select distinct * from current_memberships where org_id=".$config['org_id'];
if($_GET['sort'] && $order_by[$_GET['sort']]) {
  $sql.=' order by '.$order_by[$_GET['sort']];
}
$result = $dbh->query($sql);
my_syslog(LOG_DEBUG,'membership list: '.$sql);
if(DB::isError($result))
{
  my_syslog(LOG_ERR,'membership list isError: '.$result->getMessage());
  my_syslog(LOG_ERR,'membership list isError result: '.var_export($result));
  exit("Database error");
}
?>

<p><?=_('There are currently ')?><?=$result->numRows()?><?=_(' members.')?></p>
  <p><?=_('Click column titles to sort the table, click the member ID to edit the details of that member')?></p>
<table cellspacing="0" class="pretty">
<thead>
<tr>
  <th scope="col"><a href="<?=$PHP_SELF?>?page=memberlist&sort=id"><?=_('ID')?></a></th>
  <th scope="col"><a href="<?=$PHP_SELF?>?page=memberlist&sort=date_entered"><?=_('Date Entered')?></a></th>
  <th scope="col"><?=('Name')?> (<a href="<?=$PHP_SELF?>?page=memberlist&sort=name">F</a>|<a href="<?=$PHP_SELF?>?page=memberlist&sort=name_l">L</a>)</th>
  <th scope="col"><a href="<?=$PHP_SELF?>?page=memberlist&sort=dob"><?=_('D.O.B')?></a></th>
  <th scope="col"><a href="<?=$PHP_SELF?>?page=memberlist&sort=sex"><?=_('Sex')?></a></th>
  <th scope="col"><a href="<?=$PHP_SELF?>?page=memberlist&sort=email"><?=_('Email')?></a></th>
  <th scope="col"><a href="<?=$PHP_SELF?>?page=memberlist&sort=address"><?=_('Address')?></a></th>
  <th scope="col"><a href="<?=$PHP_SELF?>?page=memberlist&sort=home_phone"><?=_('Home Phone')?></a></th>
  <th scope="col"><a href="<?=$PHP_SELF?>?page=memberlist&sort=mobile_phone"><?=_('Mobile Phone')?></a></th>
</tr>
</thead>

<tbody>

<?php
while($row = $result->fetchrow(DB_FETCHMODE_OBJECT))
{
  print '<tr>';
  print '<td><a href="?page=edit-member&id='.htmlise($row->id).'">'.htmlise($row->id).'</a></td>';
  print '<td>'.htmlise($row->date_entered).'</td>';
  print '<td>'.htmlise($row->first_name.' '.$row->middle_name.' '.$row->last_name).'</td>';
  print '<td>'.($row->dob? htmlise($row->dob): '&nbsp;').'</td>';
  print '<td>'.($row->sex && $row->sex!=' '? htmlise($row->sex): '&nbsp;').'</td>';
  print '<td><a href="mailto:'.htmlise($row->email).'">'.htmlise($row->email).'</a></td>';
  print '<td>'.htmlise($row->address1).'<br/>'.htmlise($row->address2).'<br/>'.htmlise($row->suburb).'<br/>'.htmlise($row->state).' '.htmlise($row->postcode).'<br/>'.htmlise($row->country).'</td>';
  print '<td>'.($row->phone_home?htmlise($row->phone_home):'&nbsp;').'</td>';
  print '<td>'.($row->phone_mobile?htmlise($row->phone_mobile):'&nbsp;').'</td>';

  print '</tr>'."\n";
}
?>
</tbody>
</table>
