<?php
/**
* Deals with all database interactions for the system
*
* Uses the PEAR DB libraries to avoid database dependency
* @package database
* @author Stewart Smith
*/
require_once($config['basedir'].'/backend/syslog.inc');
/**
* Include the PEAR DB libraries.
* @link http://pear.php.net The PEAR site (hopefully the docs have improved)
*/
require_once("DB.php");

/**
* Database handle
*
* Database Handle. Used to keep one database connection for
* the running of the script.
* @global mixed $dbh
*/
$dbh = DB::connect($config['db'],array('autofree'=>true,'persistent'=>true));

if(DB::isError($dbh))
{
 my_syslog(LOG_ERR, "Failed to connect to database. dbh Message=".$dbh->getMessage());
 if($config['var_dump'])
   my_syslog(LOG_ERR, "dbh Dump:".var_export($dbh,true));

 die("Could not connect to database");
}

function sql_insert($table,$values,$dumperror=0)
{
  global $dbh, $config;

  $sql = "insert into $table (";
  $sql_values = "values (";
  $fields = array();

  foreach(array_keys($values) as $key) {
    if($values[$key])
      array_push($fields,$key);
  }
  for($i=0;$i<count($fields)-1;$i++) {
    $sql.=$fields[$i].", ";
    $sql_values.=$dbh->quote($values[$fields[$i]]).", ";
  }
  $sql.=$fields[$i];
  $sql_values.=$dbh->quote($values[$fields[$i]]).")";

  $sql.=") ".$sql_values;
  my_syslog(LOG_DEBUG,"sql_insert() SQL statement: $sql");

  $result = $dbh->query($sql);
  if (DB::isError($result)) {
    my_syslog(LOG_ERR,'sql_insert error: '.$result->getMessage());
    if($dumperror || $config['var_dump'])
       $e = var_export($result,TRUE);
    my_syslog(LOG_ERR,"sql_insert result: $e");
  }

  return $result;
}

function sql_update($table, $fields, $where)
{
  global $dbh, $config;
  $sql = "update $table set ";
  $f=array();
  foreach(array_keys($fields) as $key)
    array_push($f,$key);

  for($i=0;$i<count($fields)-1;$i++)
    $sql.=$f[$i].'='.$dbh->quote($fields[$f[$i]]).", ";
  $sql.=$f[$i].'='.$dbh->quote($fields[$f[$i]]);

  $sql.=' where '.$where;

  my_syslog(LOG_DEBUG,"sql_update() SQL: $sql");

  $result = $dbh->query($sql);
  if (DB::isError($result)) {
    my_syslog(LOG_ERR,'sql_update error: '.$result->getMessage());
    if($config['var_dump'])
      my_syslog(LOG_ERR,'sql_update result: '.var_export($result));
  }

 return $result;
}

function sql_select1($table, $fields, $where='')
{
 global $dbh, $config;
 $sql = "select ".$fields." from ".
      	$table;
 if($where)
   $sql.=" where ".$where;
 $result = $dbh->query($sql);
 my_syslog(LOG_DEBUG,'sql_select1 query: '.$sql);
 if(!DB::isError($result))
   $row = $result->fetchrow(DB_FETCHMODE_OBJECT);
 else
   {
     $row = NULL;
     my_syslog(LOG_INFO,'sql_select1 message: '.$result->getMessage());
     if($config['var_dump'])
       my_syslog(LOG_INFO,'sql_select1 result: '.var_export($result,true));
   }

 return $row;
}

function sql_select($table, $where)
{
 global $dbh;
 $sql = "select * from ".
   $table." where ".$where;
 my_syslog(LOG_ERR,'sql_select query: '.$sql);
 $result = $dbh->query($sql);
 if(!DB::isError($result))
   $row = $result->fetchrow(DB_FETCHMODE_OBJECT);
 else
   $row = NULL;

 return $row;
}

function get_permissions($member_id)
{
  global $dbh,$config;

  $sql = 'select activity,can_do from has_permission where'
    .' member_id='.$dbh->quote($member_id)
    .' and org_id='.$dbh->quote($config['org_id']);

  return $dbh->getAssoc($sql);
}

function check_permission($p,$activity)
{
  if($p[$activity] || $p['admin'])
    return 1;
  return 0;
}

function has_permission($member_id,$activity)
{
  global $dbh,$config,$permissions_cache;

  if(!$permissions_cache[$member_id])
  {
    $permissions_cache[$member_id] = get_permissions($member_id);
  }

  return check_permission($permissions_cache[$member_id],$activity);
}

function get_member_id($email)
{
  global $dbh;

  $result = sql_select1('members', 'id', 'email='.$dbh->quote($email));
  return $result->id;
}

function get_current_member_id()
{
  global $cached_current_member_id, $_SESSION;

  if($cached_current_member_id)
    return $cached_current_member_id;

  $cached_current_member_id = get_member_id($_SESSION['user']);
  return $cached_current_member_id;
}

function get_current_member()
{
  global $cached_current_member;

  if(!$cached_current_member)
  {
    $cached_current_member= get_member(get_current_member_id());
  }
  return $cached_current_member;
}

function get_member($id,$state="current_memberships")
{
  global $dbh,$config,$cached_get_member;

  if($cached_get_member[$state][$id])
    return $cached_get_member[$state][$id];

  if(count($cached_get_member[$state]) > 100)
  {
    $cached_get_member[$state]= array();
  }

  $sql = "select * from {$state} where id=".$dbh->quote($id);
  if($state=="current_memberships")
    $sql.=" and org_id=".$config['org_id'];
  $result = $dbh->query($sql);
//  my_syslog(LOG_DEBUG,'get_member query: '.$sql);
  if(!DB::isError($result))
    $row = $result->fetchrow(DB_FETCHMODE_OBJECT);
  else {
    $row = NULL;
    my_syslog(LOG_INFO, "get_member error sql: ".$sql);
    if($config['var_dump'])
      my_syslog(LOG_INFO, "result: ".var_dump($result,true));
  }

  $cached_get_member[$state][$id]= $row;
  return $row;
}

function get_member_types() {
  global $dbh,$config;
  $list = array();

  $sql = 'SELECT * FROM member_types where org_id='.$config['org_id'].' or org_id=null';
  $result = $dbh->query($sql);
  while($row = $result->fetchrow(DB_FETCHMODE_OBJECT))
    $list[] = $row;
  return $list;
}

function get_member_type($id) {
  global $dbh,$config;
  $list = array();

  $sql = 'SELECT * FROM member_types where (org_id='.$config['org_id'].' or org_id=null) and id='.$dbh->quote($id);
  my_syslog(LOG_DEBUG,'get_member_type query: '.$sql);
  $result = $dbh->query($sql);
  $row = $result->fetchrow(DB_FETCHMODE_OBJECT);

  return $row;
}

function add_qualification($name,$department,$institution)
{
 sql_insert('qualifications',
            array('name'        => $name,
                  'department'  => $department,
                  'institution' => $institution));
}

function add_org($name,$founded,$closed,$website,$contact)
{
 sql_insert('orgs', array('name'    => $name,
                          'founded' => $founded,
                          'closed'  => $closed,
                          'website' => $website,
                          'contact' => $contact));
}

function add_org_relation_type($org_id,$type,$description)
{
 sql_insert('org_relation_types', array('org_id'      => $org_id,
                                        'type'        => $type,
                                        'description' => $description));
}

function add_org_relation($org_id, $related_to_org_id, $org_relation_type_id)
{
 sql_insert('org_relations', array('org_id'               => $org_id,
                                   'related_to_org_id'    => $related_to_org_id,
                                   'org_relation_type_id' => $org_relation_type_id));
}

?>
