<?php
/**
* Deals with all logging for the system
*
* logs into syslog or database table (or both)
* @package syslog
* @author Stewart Smith
*/

// preferences for logging level. Events will be logged that have a lower priority (-1 = OFF)
/**
* Threshold for logging into the database
*
* Log messages produced by the system can either go to the UNIX
* syslog facility ({@see $logging["syslog"]}) or into a table
* in the database.
* Most users should use syslog, those who don't are either insane
* or don't have the ability to sensibly set up their syslog.
*
* This value is the threshold of what items will be logged into
* the database log table. Any log entry of a higher priority
* (lower integer value) will be inserted. A value of -1 will
* disable logging to the database
*
* Priorities are (from UNIX syslog):
* -- LOG_EMERG system is unusable
* -- LOG_ALERT action must be taken immediately
* -- LOG_CRIT critical conditions
* -- LOG_ERR error conditions
* -- LOG_WARNING warning conditions
* -- LOG_NOTICE normal, but significant, condition
* -- LOG_INFO informational message
* -- LOG_DEBUG debug-level message
* @global int $logging["database"]
*/
$logging["database"] = LOG_DEBUG;

/**
* Threshold for logging into syslog.
*
* Log messages produced by the system can either go to the UNIX
* syslog facility ({@see $logging["syslog"]}) or into a table
* in the database.
* Most users should use syslog, those who don't are either insane
* or don't have the ability to sensibly set up their syslog.
*
* This value is the threshold of what items will be logged into
* the UNIX syslog facility. Any log entry of a higher priority
* (lower integer value) will be inserted. A value of -1 will
* disable logging to syslog.
*
* Priorities are (from UNIX syslog):
* -- LOG_EMERG system is unusable
* -- LOG_ALERT action must be taken immediately
* -- LOG_CRIT critical conditions
* -- LOG_ERR error conditions
* -- LOG_WARNING warning conditions
* -- LOG_NOTICE normal, but significant, condition
* -- LOG_INFO informational message
* -- LOG_DEBUG debug-level message
* @see $logging["database"]
* @global int $logging["syslog"]
*/
$logging["syslog"] = LOG_WARNING;

define_syslog_variables();

/**
* Log Entries
*
* Priorities are (from UNIX syslog):
* -- LOG_EMERG system is unusable
* -- LOG_ALERT action must be taken immediately
* -- LOG_CRIT critical conditions
* -- LOG_ERR error conditions
* -- LOG_WARNING warning conditions
* -- LOG_NOTICE normal, but significant, condition
* -- LOG_INFO informational message
* -- LOG_DEBUG debug-level message
*
* @param int Log Priority
* @param string Message
* @author Stewart Smith
*/
function my_syslog($priority, $message)
{
        global $dbh;
        global $logging;

        if($logging["database"] > -1 && $logging["database"] >= $priority
	   && !DB::isError($dbh))
        {
	  $db_message = $dbh->quote($message);
          $db_priority = $dbh->quote($priority);
          $dbh->query("insert into log (priority,entry,username) values ($db_priority,$db_message,".$dbh->quote($_SESSION['user']).")");
        }
        if($logging["syslog"] > -1 && $logging["syslog"] >= $priority)
          syslog($priority,$message);
}

?>
