<?php

/*
 * We have different include behaviour for the pages based
 * on if the user is logged in.
 *
 * We have two 'modes' for these things:
 *    - logged in
 *    - not logged in
 *
 * If a user is logged in, functions can also be based on
 * specific permissions, but this is taken care of by
 * individual pages (i.e. they must get it right).
 *
 */
function get_page_path($page)
{
  global $logged_in;

  if($logged_in) {
    if(is_file('authenticated/'.$page.'.inc')) {
      // we're logged in, and authenticated page exists... so...
      $page_inc = 'authenticated/'.$page.".inc";
    } else {
      if(is_file('unauthenticated/'.$page.'.inc')) {
	// try the unauthenticated page, as there isn't an auth version
	$page_inc = 'unauthenticated/'.$page.'.inc';
      } else {
	// the file doesn't exist, and we're authenticated
	my_syslog(LOG_WARNING,"Unknown page passed while auth, defaulting to index: $page");
	$page = 'index';
	$page_inc = 'unauthenticated/'.$page.'.inc';
      }
    }
  } else {
    if(is_file('unauthenticated/'.$page.'.inc')) {
    // we're not authenticated, and not logged in, but the page exists.
      $page_inc = 'unauthenticated/'.$page.'.inc';
    } else {
      if(is_file($page.'.inc'))	{
	// The inc file is in the current dir
	$page_inc = $page.'.inc';
      } else {
	// default back to index
	my_syslog(LOG_WARNING,"Unknown page passed while unauth, defaulting to index: $page");
	$page = 'index';
	$page_inc = 'unauthenticated/index.inc';
      }
    }
  }

  return $page_inc;
}

function show_menu_r(&$user_permissions,$menu)
{
  global $logged_in;
  
  foreach ($menu as $page) {  
    if(isset($page['category'])) {
      $s = show_menu_r($user_permissions,$page['items']);
      if($s) {
	$str.='<li>';
	if(isset($page['page']))
	  $str.='<a href="?page='.$page['page'].'">';
	$str.=$page['category'];
	if(isset($page['page']))
	  $str.='</a>';
	$str.='<ul>'.$s.'</ul></li>';
      } else if(isset($page['page'])) {
	if(!$page['logged_in'] || ($logged_in && $page['logged_in']))
	  $str.='<li><a href="?page='.$page['page'].'">'.
	    $page['category'].'</a></li>';
      }
    } else if(!isset($page['activity'])
	      || check_permission($user_permissions, $page['activity'])
	      ) {
      if(!$page['logged_in'] || ($logged_in && $page['logged_in']))
	$str.='<li><a href="?page='.
	  $page['page'].'">'.
	  $page['title'].'</a></li>';
    }
  }
  return $str;
}

function show_menu($menu)
{
  $user_permissions= get_permissions(get_member_id($_SESSION['user']));

  return show_menu_r($user_permissions,$menu);
}

/* Dump an array, but preformatted HTML plzkthxbye.  */
function print_pre($array) {
  echo "<pre>\n";
  print_r($array);
  echo "</pre>\n";
  return;
}

/**
 * strtotime with configurable date and time input format. Hey ho!
 * Note that strptime is not implemented on windows.
 * Wants:
 * @date       Date - formatted as per datefmt cfg.
 * @time       Time - formatted as per timefmt cfg.
 */
function my_strtotime($date, $time) {
  global $config;

  if(!function_exists("strptime")) {
    my_syslog(LOG_CRIT,"The strptime() function is not implemented. my_strtotime() will always return 0.");
    return 0;
  }
  $t = strptime($date .' '. $time, $config['date_format'] .' '. $config['time_format'] );
  if( $t == NULL ) return 0;
  return mktime($t['tm_sec'], $t['tm_min'], $t['tm_hour'], $t['tm_mon']+1, $t['tm_mday'], $t['tm_year']+1900);
}

/**
 * Removes mails with empty recipient from the mail queue, so it'll run without whining.
 * Not sure how they end up in there to begin with, to be honest. Oh well!
 */
function fix_mail_queue() {
  global $dbh;
  $sql= "/* memberdb::util::fix_mail_queue() */ DELETE FROM mail_queue WHERE recipient=".$dbh->quote( 's:0:"";' );
  return $dbh->query($sql);
}

?>
