<?php
/**
* Main page generator
*
* This page generates all of the pages in the site.
* It takes care of authentication and all that stuff
* and then includes the actual page that the user
* requested
*
* @author Stewart Smith
*/

function memberdb_version()
{
  return "0.6-bzr";
}

function memberdb_config_error($error)
{
    ?>
  <html>
    <head>
      <title><?php echo _('MemberDB Configuration Error')?></title>
    </head>
    <body>
    <h1><?php echo _('MemberDB Configuration Error')?></h1>
    <p><?php echo $error ?></p>

  </html>

    <?php
}

$time = microtime();
$time = explode(' ', $time);
$time = $time[1] + $time[0];
$begintime = $time;

/**
 * Include site configuration
 *
 * @see config.inc
 */
if(!$config)
{
  if(!file_exists('config.inc'))
  {
    memberdb_config_error(_('Could not find the config.inc configuration file.'));
    exit(0);
  }
  include 'config.inc';
}


if($_SERVER['PATH_INFO']=='/global.css') {
  header("Content-Type: text/css; charset=utf-8");
  include $config['basedir'].'/global.css';
  exit(0);
} else if($_SERVER['PATH_INFO']=='/site.css') {
  header("Content-Type: text/css; charset=utf-8");
  include $config['basedir'].'/site-look/site.css';
  exit(0);
}

function magicQuotesRemove(&$array) {
   if(!get_magic_quotes_gpc())
       return;
   foreach($array as $key => $elem) {
       if(is_array($elem))
           magicQuotesRemove($elem);
       else
           $array[$key] = stripslashes($elem);
   }
}

magicQuotesRemove($_GET);
magicQuotesRemove($_POST);
magicQuotesRemove($_COOKIE);


// I18N support information here
$language = 'en';
putenv("LANG=$language"); 
putenv("LANGUAGE=$language"); 
putenv("LC_ALL=$language"); 
setlocale(LC_ALL, $language);

// Set the text domain as 'messages'
$domain = 'messages';
bindtextdomain($domain, './locale');
textdomain($domain);

/**
 * List of pages, categories and associated permissions
 *
 * @see pages.inc
 */
include 'pages.inc';

/**
* Database access functions
*
* @see database
*/
include 'backend/database.inc';

/**
 * Token functions
 *
 * Used to have random (preserved) tokens for things
 * like signing up, recovering passwords.
 *
 * @see token
 */
include 'backend/token.inc';

/**
* Include Authentication code
*
* @see authentication.inc
*/
include 'backend/authentication.inc';

/**
 * Include HTML output functions
 *
 * @see backend/output.inc
 */
include 'backend/output.inc';

/**
 * Include Forms Code
 *
 * @see backend/forms.inc
 */
include 'backend/forms.inc';

/**
 * Include the site_messages code. This accesses the DB and will
 * do appropriate substitutions for sending mail.
 *
 * @see backend/site_messages.inc
 */
include 'backend/site_messages.inc';

/**
 * Include a bunch of useful functions that don't belong anywhere else
 *
 * @see backend/utils.inc
 */
include 'backend/utils.inc';

$current_schema_version= 5;
$dbver= sql_select1('schema_version','version');
$dbver= $dbver->version;
if($dbver < $current_schema_version)
{
	include 'site-look/db_upgrade_needed.inc';
	exit();
}
else if($dbver > $current_schema_version)
{
	include 'site-look/db_too_new.inc';
	exit();
}
$org= sql_select1('orgs','*','id='.$dbh->quote($config['org_id']));

// Start the session
session_start();

$logged_in = $_SESSION['logged_in'];

if($_GET['page']=='confirm_membership')
     $_POST['function']='confirm_membership';

if($logged_in && !get_current_member() && get_member(get_current_member_id(),'has_had_membership')) {
/*
 * we are logged in, but membership has lapsed (or has not been approved)
 */

  $HTTP_GET_VARS['page']='renew_membership';
}

switch($_POST['function']) {
 case 'login':
   $_SESSION['user'] = $_POST['user'];

   $logged_in = login($_POST['user'],$_POST['password']);

   $_SESSION['logged_in'] = $logged_in;

   if(!$logged_in) {
     $login_error = _("Invalid username or password");
     unset($_SESSION['user']);
     unset($_SESSION['logged_in']);
   }
   break;

 case 'logout':
   $logged_in = 0;
   unset($_SESSION['user']);
   unset($_SESSION['logged_in']);
   $HTTP_POST_VARS['function']='login';
   break;

 case 'confirm_membership':
   include 'backend/functions/confirm_membership.inc';
   break;

 case 'approve_members':
   include 'backend/functions/approve_members.inc';
   break;

 case '':
   break;
 default:
   echo '<div class="error"><p>'._('Unknown Function!').' '.$_POST['function'].
     '</p></div>';
   my_syslog(LOG_WARNING,"Unknown function submitted - ".$_POST['function']);
   break;
}

/* if after performing the function we still don't have a page
   to view, then we set one - the default index page
*/
if(!$page) {
  $page = $HTTP_GET_VARS['page'];
  if($page=='') {
    $page = explode('/',$_SERVER['PATH_INFO']);
    $page = $page[1];
  }
  if($page=='') $page = 'index';
}

if(!eregi("^[[:alpha:][:digit:]\_-]+$",$page)) {
  $page= 'index';
  my_syslog(LOG_WARNING,"Invalid page requested: $page");
}

// work out what page to load
$page_inc= get_page_path($page);

$have_already_done_this_bloody_thing= 0;

function headers($param)
{
  global $permissions_pages, $page_param,$logged_in,$login_error,$page;
  global $have_already_done_this_bloody_thing;
  global $config;

  if($have_already_done_this_bloody_thing)
    return;

  $have_already_done_this_bloody_thing=1;

  $page_param = $param;

  header("Content-Type: text/html; charset=utf-8");
  header("X-Pants: off");
  header("X-Diebold: false");
?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN" "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en">
 <head>
  <title><?=$param['title']?></title>
  <meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
  <link rel="stylesheet" type="text/css" href="<?=$config['baseurl']?>/global.css" />
  <?php @include 'site-look/html-head-extras.inc'; ?>
  <?=$param['head_extras']?>
  <script type="text/javascript" src="lib/sorttable.js"></script>
 </head>
<body>

<?php include 'site-look/header.inc'; ?>

<div class="container">
<?php
   if(!$param['menu']) {
     ?>
     <div class="mini-login">
     <p>You are logged in as <?=$_SESSION['user']?></p>
     </div>
     <div class="menulist">
     <ul class="menulist">
     <li><a href="?page=index">Home</a></li>
     </ul>
     </div>
     <?php
     } else {
     echo '<div class="menu">';
      print '<ul class="menu">';
      print show_menu($permissions_pages); // do menu stuff
      print '</ul>';
   }

if($page_param['menu']) {
  echo '<div class="login">';
  if(!$logged_in) {
    $fields = array('function'=>array('type'=>'hidden',
				      'default'=>'login'),
		    'user'=>array('type'=>'text',
				  'label'=>'Email Address',
				  'required'=>true,
				  'valid'   =>array(
					 'preg_match("/\w+/", $val)'
					 =>'Enter the email address that you used to sign up.'
					 ),
				  ),
		    'password'=>array('type'=>'password',
				  'label'=>'Password',
				  'required'=>true,
				  'valid'   =>array(
					 'preg_match("/\w+/", $val)'
					 =>'Enter your password'
					 ),
				  ),
		    'html'=>array('type'=>'html',
				  'html'=>'<a href="?page=recover-password">Forgot your password?</a>'
				  ),
		    );


    echo '<h2>'._('Login').'</h2>';

    if($login_error)
      echo '<div class="error"><p>'.$login_error.'</p></div>';

    $errors = validate($fields, $HTTP_POST_VARS);

    output_form($fields,"login",
		"Login","post",
		($page=='reset-password')?'?page=index':$PHP_SELF
		);

  } else {
    echo '<p>'._('You are logged in as').' '.$_SESSION['user'].'</p>';

    $fields = array('function'=>array('type'=>'hidden',
				      'default'=>'logout',
				      'value'=>'logout'),
		    );
    output_form($fields,"logout",
		"Logout","post",'');

  }
    ?>
      </div>
      </div>
      </div> <!-- menu -->
<?php
	  }
 if(!$param['menu'])
   echo '<div class="bodyformini">';
 else
   echo '<div class="body">';
}
include $page_inc;

if($config['send_emails'])
  // Run util func to remove invalid msgs (no recipient) from queue.
  @fix_mail_queue();
  @$mail_queue->sendMailsInQueue(10);
?>
</div>

<div class="copyright">
<hr/>
<?php include $config['basedir'].'/site-look/footer.inc';?>
<p>
<?php
 $time = microtime();
$time = explode(" ", $time);
$time = $time[1] + $time[0];
$endtime = $time;
$totaltime = ($endtime - $begintime);
echo _('Page generated in ').$totaltime. _(' seconds.');
?>
</p>
<p>This site is running <a href="http://www.flamingspork.com/projects/memberdb/">MemberDB</a> <?=memberdb_version()?> &copy;2002-2008 Stewart Smith. Report bugs at <a href="http://bugzilla.flamingspork.com">http://bugzilla.flamingspork.com/</a>.</p>
</div>
<script type="text/javascript">
sortables_init();
</script>
</body>
</html>
