alter table log add username varchar(50);

create table site_messages (
	name varchar(50),
	mail_to text,
	mail_cc text,
	message text
);

CREATE INDEX site_messages_name_idx ON site_messages(name);

CREATE or replace view has_had_membership AS
	select distinct
	  members.*,
	  orgs.id as org_id,
	  orgs.name as org_name,
	  member_types.id as member_type_id,
	  member_types.type as member_type
	from
	  members,
	  orgs,
	  org_members,
	  member_types
	where
	      members.id = org_members.member_id
	  AND org_members.org_id = orgs.id
	  AND org_members.member_type_id = member_types.id
	  AND member_types.org_id = orgs.id 
	  and (
		--
		-- you are a current member if your membership:
		-- - period started before now, and ends after now
		-- - never ends, always existed
		-- - started before now, never ends
		-- - started at unknown, ends after now
			(org_members.start_date < now())
		     or (org_members.start_date is null)
		     or (org_members.start_date<now())
		     or (org_members.start_date is null)
		    )
	  AND member_types.validates_membership = true
	  AND not exists (select member_id from org_members,member_types
		where org_members.member_type_id = member_types.id
		and org_members.member_id = members.id
		and member_types.revokes_membership = true
		and member_types.validates_membership = false
		and (
		--
		-- you are a current member if your membership:
		-- - period started before now, and ends after now
		-- - never ends, always existed
		-- - started before now, never ends
		-- - started at unknown, ends after now
			(org_members.start_date < now()
			  and org_members.expiry > now())
		     or (org_members.start_date is null
			  and org_members.expiry is null)
		     or (org_members.start_date<now()
			  and org_members.expiry is null)
		     or (org_members.start_date is null
			  and org_members.expiry>now())
		    )
		)
	;

create table members_old_details (
	id integer not null,
	date_entered datetime,
	first_name varchar(50) not null,
	middle_name varchar(50),
	last_name varchar(50),
	DOB date default NULL,
	sex char(1) default NULL,
	address1 varchar(50) default NULL,
	address2 varchar(50) default NULL,
	suburb varchar(50) default NULL,
	postcode varchar(10) default NULL,
	state varchar(50) default NULL,
	country varchar(20) default NULL,
	email varchar(50) default NULL ,
	phone_home varchar(25) default NULL,
	phone_mobile varchar(25) default NULL,
	CONSTRAINT "members_old_details_id_fkey" FOREIGN KEY (id) references members(id) on update restrict,
	CONSTRAINT "members_old_details_pkey" PRIMARY KEY (id,date_entered)
);

create or replace view org_memberships as select member_id, om.org_id, mt.type as type, description, start_date, expiry,validates_membership, revokes_membership from org_members as om LEFT JOIN member_types as mt ON mt.id=member_type_id;
